#include "hourglass.h"
#include "bossa.h"
#include "user_stub_Linux.h"

#define MAX_MOUNTED 25
static char *mounted[MAX_MOUNTED];
static int mounted_ctr = 0;

#define BOSSA_ID_NUM 0xCE
#define IOCTL_ATTACH                 _IOW(BOSSA_ID_NUM, 255, int)
#define BOSSA_DEVICE_NAME ""

extern char *new_root_scheduler;

void bossa_mount_root(char *parent, int num_args, int *args) {
  fprintf(stderr,"adding root %s\n",parent);
  mount_root(parent, num_args, args);
}

void bossa_mount(char *parent, char *child, int num_args, int *args) {
  int i;
  for (i=0; i!=mounted_ctr; i++) {
    if (strcmp(child,mounted[mounted_ctr]) == 0) {
      fprintf(stderr,"Cannot doubly mount scheduler %s\n",child);
      bossa_cleanup();
      exit(0);
    }
  }
  if (mounted_ctr < MAX_MOUNTED) {
    mounted[mounted_ctr] = child;
    mounted_ctr++;
  }
  else {
      fprintf(stderr,"Too many mounted schedulers %s\n",child);
      bossa_cleanup();
      exit(0);
  } 
  fprintf(stderr,"adding parent %s child %s\n",parent,child);
  mount(parent, child, num_args, args);
}

void bossa_cleanup() {
  int i;
  // unmount process schedulers
  for(i=0;i<mounted_ctr;i++) {
    unmount(mounted[i]);
  }
  // unmount virtual scheduler, if any
  if (new_root_scheduler) unmount(new_root_scheduler);
}

void attach(const char *policy, int *args) {
  int res;
  int fd;
  char *dir;
  
  if (policy == NULL) return;

  if (strcmp(policy,"Linux") == 0) {
    Linux_sched_setscheduler(args[1], args[2], args[3]);
    if (args[2] == BOSSA_SCHED_OTHER) {
      Linux_set_priority(args[1], args[4]);
    }
  }
  else {
    dir = (char *)malloc(strlen("/dev/bossa/bossa_") + strlen(policy) + 1);
    strcpy(dir,"/dev/bossa/bossa_");
    strcat(dir,policy);
    fd = open(dir, O_WRONLY);
    res = ioctl(fd,IOCTL_ATTACH,(char *)args);
    close(fd);
    if (res < 0) {
      perror("attach");
      bossa_cleanup();
      exit(0);
    }
  }
}

void detach(const char *policy) {
  if (policy == NULL) return;
  if (strcmp(policy,"Linux") != 0)
    Linux_attach(0,BOSSA_SCHED_OTHER,0,20);
}
