/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"

void *xmalloc (size_t size)
{
  void *v = malloc (size);
  if (!v) {
    printf ("whoops, malloc failure -- exiting\n");
    exit (-1);
  }
  return v;
}

static us_time get_timeval_us (void)
{
  struct timeval tv;
  us_time t;
  
  gettimeofday (&tv, NULL);
  t = tv.tv_usec + (1000000 * tv.tv_sec);
  return t;
}

static double my_fabs (double d)
{
  if (d < 0) {
    return -d;
  } else {
    return d;
  }
}

double find_mhz (void)
{
  double last_freq, freq = 0;
  int i;
  int good = 0;
  us_time us_start, us_stop, us_diff;
  cycle_time cycle_start, cycle_stop, cycle_diff;

  for (i=0; i<50; i++) {

    // estimate MHz

    us_start = get_timeval_us ();
    cycle_start = rdtsc ();

    usleep (1000);

    us_stop = get_timeval_us ();
    cycle_stop = rdtsc ();

    us_diff = us_stop - us_start;
    cycle_diff = cycle_stop - cycle_start;

    last_freq = freq;
    freq = cycle_diff / us_diff;
    
    // wait for three successive values within 1 MHz

    if (my_fabs (last_freq - freq) < 1.0) {
      good++;
      if (good == 3) goto success;
    } else {
      good = 0;
    }
  }
  
  printf ("couldn't determine MHz of this processor: try reducing\n");
  printf ("machine load.\n");
  exit (-1);
  
 success:

  return freq;
}

